# frozen_string_literal: true

# CarrierWave Secure - Licensed under LGPL-3.0

module CarrierWave
  module Secure
    # Gives `encrypt` and `decrypt` powers to CarrierWave::SanitizedFile instances, using decorator
    # pattern.
    class AbstractFile < CarrierWave::SanitizedFile
      def initialize(decorated)
        @decorated = decorated
      end

      # DELEGATED METHODS

      def filename
        @decorated.filename
      end

      def original_filename
        if @decorated.respond_to?(:original_filename)
          @decorated.original_filename
        elsif @decorated.respond_to?(:path) && @decorated.path
          File.basename(@decorated.path)
        end
      end

      def basename
        @decorated.basename
      end

      def extension
        @decorated.extension
      end

      def size
        @decorated.size
      end

      def exists?
        @decorated.exists?
      end

      def delete
        @decorated.delete
      end

      def content_type
        @decorated.content_type
      end

      def content_type=(new_content_type)
        @decorated.content_type = new_content_type
      end

      def empty?
        @decorated.nil? || size.nil? || (size.zero? && !exists?)
      end

      def path
        nil
      end

      private

      def cipher
        CarrierWave::Secure.cipher
      end
    end
  end
end
