# frozen_string_literal: true

# CarrierWave Secure - Licensed under LGPL-3.0

require 'openssl'

module CarrierWave
  module Secure
    # Encrypts and decrypts data
    class Cipher
      AUTH_TAG_LENGTH = 16
      DEFAULT_KEY_LENGTH = 256 # bits

      def initialize(key)
        @key = key
      end

      def encrypt(data)
        ssl = OpenSSL::Cipher::AES.new(key_length_bits, :gcm)
        ssl.encrypt
        ssl.key = @key
        iv = ssl.random_iv
        ssl.iv = iv
        ssl.update(data) << ssl.final << iv << ssl.auth_tag
      end

      def decrypt(data)
        ssl = OpenSSL::Cipher::AES.new(key_length_bits, :gcm)
        ssl.decrypt
        ssl.key = @key
        ssl.auth_tag = data.slice!(-AUTH_TAG_LENGTH..-1)
        ssl.iv = data.slice!(-ssl.iv_len..-1)
        ssl.update(data) << ssl.final
      end

      def random_key
        OpenSSL::Cipher::AES.new(key_length_bits, :gcm).random_key
      end

      private

      def key_length_bits
        @key && @key.length * 8 || DEFAULT_KEY_LENGTH
      end
    end
  end
end
